package com.care.mqtt.service;


import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.care.bigscreen.service.BigScreenService;
import com.care.common.cache.RedisUtil;
import com.care.common.constant.Constants;
import com.care.common.entity.CareDevice;
import com.care.common.entity.CareEventOrder;
import com.care.common.entity.CareMqttMsg;
import com.care.common.enums.MqttMsgStatusEnum;
import com.care.common.enums.OrderStatusEnum;
import com.care.common.enums.OrderTypeEnum;
import com.care.common.service.CareDeviceService;
import com.care.common.service.CareEventOrderService;
import com.care.common.service.CareMqttMsgService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Set;


/**
 * 使用redis轮训方案实现延迟触发
 * @Author: stw
 * @Date: 2021/8/14
 * @Desc:
 */
@Service
public class MqttMsgRedisService {
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareMqttMsgService careMqttMsgService;
    @Resource
    private CareEventOrderService careEventOrderService;
    @Resource
    private MqttMsgService mqttMsgService;
    @Resource
    private BigScreenService bigScreenService;
    @Resource
    private RedisUtil redisUtil;


    /**
     * 读取数据库mqtt msg表，把有延迟标记、未处理的消息生成调redis结构放入redis
     */
    public void init() {

        //查询mqtt msg 表有延迟标记的未处理消息，放入线程池,等待延时时间结束执行创建工单
        QueryWrapper<CareMqttMsg> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.lambda().eq(CareMqttMsg::getStatus, MqttMsgStatusEnum.NO_HANDLE.getValue()).gt(CareMqttMsg::getDelayTime,0)
        .orderByAsc(CareMqttMsg::getCreateTime);

        List<CareMqttMsg> mqttMsgList = careMqttMsgService.list(queryWrapper2);
        if (CollUtil.isNotEmpty(mqttMsgList)) {
            for(CareMqttMsg careMqttMsg : mqttMsgList){

                Long now = System.currentTimeMillis();
                Long createTime = careMqttMsg.getCreateTime().getTime();
                Long delayTime = careMqttMsg.getDelayTime() * 60 * 1000L;
                Long execTime = createTime + delayTime;

                if(execTime <= now) { //立即执行
                    exe(careMqttMsg);
                    continue;
                }
                addRedis(careMqttMsg,execTime);
            }

        }
    }



    /**
     * 把延迟消息放入redis
     * @param careMqttMsg
     * @param execTime
     */
    public void addRedis(CareMqttMsg careMqttMsg, long execTime){

        String msg = String.valueOf(careMqttMsg.getId());
        redisUtil.zSetAdd(Constants.MQTT_MSG_LAG_KEY, msg, execTime);
    }

    /**
     * 轮训redis
     */
    public void rotationRedis() {
        long startTime = 0L;
        long endTime = System.currentTimeMillis();

        //查询符合时间条件下的集合
        Set set = redisUtil.zSetRangeByScore(Constants.MQTT_MSG_LAG_KEY, startTime, endTime);

        if (CollUtil.isNotEmpty(set)) {
            //具体业务操作
            set.forEach(item -> {
                CareMqttMsg careMqttMsg = careMqttMsgService.getById(Long.parseLong(String.valueOf(item)));
                exe(careMqttMsg);
            });

            //移除集合
            redisUtil.zSetRemoveRangeByScore(Constants.MQTT_MSG_LAG_KEY, startTime, endTime);
        }
    }

    /**
     *  执行创建工单动作
     * @param careMqttMsg
     */

    private void exe(CareMqttMsg careMqttMsg){
        String status = this.careMqttMsgService.getById(careMqttMsg).getStatus();
        if(!MqttMsgStatusEnum.NO_HANDLE.getValue().equals(status)){ //不是未处理，不执行。
            return;
        }

        CareDevice careDevice = careDeviceService.getById(careMqttMsg.getDevId());
        CareEventOrder order = mqttMsgService.getNewOrder(careDevice);
        order.setCreateTime(careMqttMsg.getCreateTime());
        order.setModifyTime(careMqttMsg.getCreateTime());
        order.setStatus(OrderStatusEnum.TODO.getValue());
        if ("FallDown".equals(careMqttMsg.getType())) { //跌到
            order.setOrderType(OrderTypeEnum.DI_EDAO.getValue());
        } else if("StayTooLong".equals(careMqttMsg.getType())){ //久滞
            order.setOrderType(OrderTypeEnum.JIU_ZHI.getValue());
        }

        //判断房屋是否有未完成的告警事件,不区分类型
        QueryWrapper<CareEventOrder> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.lambda().eq(CareEventOrder::getHouseId,careDevice.getHouseId()).and(
                wrapper-> wrapper.eq(CareEventOrder::getStatus, OrderStatusEnum.TODO.getValue())
                        .or().eq(CareEventOrder::getStatus,OrderStatusEnum.DOING.getValue()));

        CareEventOrder orderDb = careEventOrderService.getOne(queryWrapper2);
        if(orderDb != null) { //有未完成的告警事件，不再生成新的告警工单，只作为子事件插入到当前工单的历史记录中。
            careEventOrderService.saveHisOrder(order,orderDb);
            //通知页面
            this.bigScreenService.pushRtEventFlag(order.getStationId().toString(),order.getId().toString(),"orderUpdate");
            this.bigScreenService.pushRtEventFlag(order.getStationId().toString(),order.getId().toString(),"bigscreenRefresh");
        }  else { //无未完成的告警事件
            //生成新的工单
            careEventOrderService.saveOrder(order);
            //通知页面
            this.bigScreenService.pushRtEventFlag(order.getStationId().toString(),order.getId().toString(),"orderAdd");
            this.bigScreenService.pushRtEventFlag(order.getStationId().toString(),order.getId().toString(),"bigscreenRefresh");
        }

        careMqttMsg.setStatus(MqttMsgStatusEnum.HANDLED.getValue());
        careMqttMsg.setModifyTime(new Date());
        this.careMqttMsgService.updateById(careMqttMsg);
    }

}
